require 'studio_game/player'

module StudioGame

	describe Player do
		
		before do
			@initial_health = 150
			@player = Player.new("joe", @initial_health)
		end

		it "has a capitalized name" do
			expect(@player.name).to eq("Joe")
		end

		it "has an initial health" do
			expect(@player.health).to eq(150)
		end

		it "has a string representation" do
			@player.found_treasure(Treasure.new(:hammer, 50))
			@player.found_treasure(Treasure.new(:hammer, 50))

			expect(@player.to_s).to eq("I'm Joe with health = 150, score = 250, and points = 100.")
		end

		it "increases health by 15 when w00ted" do
			@player.w00t
			expect(@player.health).to eq(@initial_health + 15)
		end

		it "decreases health by 10 when blammed" do
			@player.blam
			expect(@player.health).to eq(@initial_health - 10)
		end

		it "can be created from a CSV string" do
	  		player = Player.from_csv("larry,150")

	  		expect(player.name).to eq("Larry")
	  		expect(player.health).to eq(150)
		end

		context "created with a default health" do
			before do
				@player = Player.new("larry")
			end

			it "has a health of 100" do
				expect(@player.health).to eq(100)
			end

		end

		context "with a health greater than 100" do

			before do
				@initial_health = 150
				@player = Player.new("larry", @initial_health)
			end

			it "is strong" do
				expect(@player).to be_strong
			end
		end

		context "with a health of 100 or less" do

			before do 
				@initial_health = 100
				@player = Player.new("fart", @initial_health)
			end

			it "is wimpy" do
				expect(@player.strong?).to eq(false)
			end
		end

		context "in a collection of players" do
			
			before do
				@player = Player.new("moe", 100)
	    		@player2 = Player.new("larry", 200)
	    		@player3 = Player.new("curly", 300)

	    		@players = [@player, @player2, @player3]

			end

			it "is sorted by decreasing score" do
				expect(@players.sort).to eq([@player3, @player2, @player])
			end
		end

		it "computes points as the sum of all treasure points" do
	  		expect(@player.points).to eq(0)

	  		@player.found_treasure(Treasure.new(:hammer, 50))

	  		expect(@player.points).to eq(50)

	  		@player.found_treasure(Treasure.new(:crowbar, 400))

	  		expect(@player.points).to eq(450)

	  		@player.found_treasure(Treasure.new(:hammer, 50))

	  		expect(@player.points).to eq(500)
		end

		it "yields each found treasure and its total points" do
	  		@player.found_treasure(Treasure.new(:skillet, 100))
	  		@player.found_treasure(Treasure.new(:skillet, 100))
	  		@player.found_treasure(Treasure.new(:hammer, 50))
	  		@player.found_treasure(Treasure.new(:bottle, 5))
	  		@player.found_treasure(Treasure.new(:bottle, 5))
	  		@player.found_treasure(Treasure.new(:bottle, 5))
	  		@player.found_treasure(Treasure.new(:bottle, 5))
	  		@player.found_treasure(Treasure.new(:bottle, 5))

	  		yielded = []
	  		@player.each_found_treasure do |treasure|
	    		yielded << treasure
	  		end

	  		expect(yielded).to eq([
	    		Treasure.new(:skillet, 200),
	    		Treasure.new(:hammer, 50),
	    		Treasure.new(:bottle, 25)
	 		])
		end



	end
end